@@ -9,8 +9,10 @@
 
 **********************************************************************/
 
-#include "internal.h"
+#include "internal/inits.h"
+#include "ruby.h"
 #include "builtin.h"
+static void Init_builtin_prelude(void);
 #include "prelude.rbinc"
 
 #define CALL(n) {void Init_##n(void); Init_##n();}
@@ -18,6 +20,8 @@
 void
 rb_call_inits(void)
 {
+    CALL(default_shapes);
+    CALL(Thread_Mutex);
 #if USE_TRANSIENT_HEAP
     CALL(TransientHeap);
 #endif
@@ -35,7 +39,6 @@ rb_call_inits(void)
     CALL(String);
     CALL(Exception);
     CALL(eval);
-    CALL(safe);
     CALL(jump);
     CALL(Numeric);
     CALL(Bignum);
@@ -44,10 +47,12 @@ rb_call_inits(void)
     CALL(Hash);
     CALL(Struct);
     CALL(Regexp);
+    CALL(pack);
     CALL(transcode);
     CALL(marshal);
     CALL(Range);
     CALL(IO);
+    CALL(IO_Buffer)
     CALL(Dir);
     CALL(Time);
     CALL(Random);
@@ -56,26 +61,54 @@ rb_call_inits(void)
     CALL(Proc);
     CALL(Binding);
     CALL(Math);
+    CALL(GC);
     CALL(Enumerator);
+    CALL(Ractor);
     CALL(VM);
     CALL(ISeq);
     CALL(Thread);
+    CALL(Fiber_Scheduler);
     CALL(process);
     CALL(Cont);
     CALL(Rational);
     CALL(Complex);
+    CALL(MemoryView);
     CALL(version);
+    CALL(vm_trace);
     CALL(vm_stack_canary);
+    CALL(ast);
     CALL(gc_stress);
+    CALL(shape);
 
     // enable builtin loading
     CALL(builtin);
+}
 
-    CALL(GC);
-    CALL(IO_nonblock);
-    CALL(ast);
-    CALL(vm_trace);
-    CALL(pack);
-    load_prelude();
+void
+rb_call_builtin_inits(void)
+{
+#define BUILTIN(n) CALL(builtin_##n)
+    BUILTIN(gc);
+    BUILTIN(ractor);
+    BUILTIN(numeric);
+    BUILTIN(io);
+    BUILTIN(dir);
+    BUILTIN(ast);
+    BUILTIN(trace_point);
+    BUILTIN(pack);
+    BUILTIN(warning);
+    BUILTIN(array);
+    BUILTIN(kernel);
+    BUILTIN(timev);
+    BUILTIN(thread_sync);
+    BUILTIN(yjit);
+    BUILTIN(nilclass);
+    BUILTIN(marshal);
+#if USE_MJIT
+    BUILTIN(mjit);
+    BUILTIN(mjit_c);
+    BUILTIN(mjit_compiler);
+#endif
+    Init_builtin_prelude();
 }
 #undef CALL
